unit uMain;

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	ExtCtrls, Globe3, Buttons, StdCtrls, Db, TGLayerDS, Grids,
	DBGrids, DBCtrls, tgutils;

type
	TForm1 = class( TForm )
		Globe1 : TGlobe3;
		Panel1 : TPanel;
		SpeedButton1 : TSpeedButton;
		SpeedButton2 : TSpeedButton;
		SpeedButton3 : TSpeedButton;
		OpenDialog1 : TOpenDialog;
		SpeedButton4 : TSpeedButton;
		Panel2 : TPanel;
		DBGrid1 : TDBGrid;
		DataSource1 : TDataSource;
		Panel3 : TPanel;
		btnZoomOut : TSpeedButton;
		btnZoomIn : TSpeedButton;
		Splitter1 : TSplitter;
		btnTransparent : TSpeedButton;
		SpeedButton7 : TSpeedButton;
		btnAbout: TSpeedButton;
    Timer1: TTimer;
    OpenDialog2: TOpenDialog;
    Globe2: TGlobe3;
    GlobeLayerDS1: TGlobeLayerDS;
    GlobeLayerDS1Index: TIntegerField;
    GlobeLayerDS1Title: TStringField;
    GlobeLayerDS1Value: TFloatField;
    GlobeLayerDS1FeatureID: TIntegerField;
    GlobeLayerDS1ID: TIntegerField;
    GlobeLayerDS1RowID: TIntegerField;
    GlobeLayerDS1Hidden: TBooleanField;
    GlobeLayerDS1Selected: TBooleanField;
		procedure FormCreate( Sender : TObject );
		procedure Globe1Render( Sender : TObject );
		procedure Globe1MouseMove( Sender : TObject; Shift : TShiftState; X,
			Y : Integer );
		procedure SpeedButton1Click( Sender : TObject );
		procedure Globe2Paint( Sender : TObject );
		procedure Globe1Paint( Sender : TObject );
		procedure DBGrid1CellClick( Column : TColumn );
		procedure btnZoomClick( Sender : TObject );
		procedure btnViewTypeClick( Sender : TObject );
		procedure DBGrid1TitleClick( Column : TColumn );
		procedure btnAboutClick(Sender: TObject);
		procedure Timer1Timer(Sender: TObject);
	private
		{ Private declarations }
	public
		{ Public declarations }
		procedure Idle(Sender: TObject; var Done: Boolean);
	end;

var
	Form1 : TForm1;
	LayerBrush : TBrush;
	MyObject: TGlobeObject;
	ptList, ptList2 : TGeoDataObject;
	giUpdateGlobe : integer;

implementation

{$R *.DFM}

Uses uAbout;

{------------------------------------------------------------------------------
	TForm1.FormCreate
------------------------------------------------------------------------------}
procedure TForm1.FormCreate( Sender : TObject );
var
	ALayer : TGlobeLayer;
	sAppPath : string;
begin
	sAppPath := ExtractFilePath( Application.ExeName );

	Globe1.ProfileName := sAppPath + 'tglobe.prf';

	LayerBrush := TBrush.Create;
	LayerBrush.Style := bsClear;

	Globe2.TextureFilename := sAppPath + 'world.bmp';
	Globe2.TextureBackground := True;
	if not Globe2.TextureBackground then
		Globe2.DrawGrid := dgBehind;

	ALayer := Globe1.LayerNew('Test');
	with TBitmapObject.Create( ALayer, 'WorldMap', 0, nil ) do
	begin
		{ bitmap is 640 x 320 so center about the origin }
		Origin := PointLL( -320 * GU_MINUTE, 160 * GU_MINUTE );
		BMPFilename := sAppPath + 'world.bmp';
		GlobeUnitsPerPixel := GU_MINUTE;
	end;

	{ Zoom the globes out to their extents }
	Globe2.Altitude := 0;
	Globe1.Altitude := 0;

	{ Switch on the Layer dataset }
	if Globe1.LayerCount > 0 then
	begin
		GlobeLayerDS1.Layer := Globe1[0];
		GlobeLayerDS1.Active := True;
	end;

	{ Create an animated Layer }
	ALayer := Globe1.LayerNew( 'Animated' );
	ALayer.Animated := True;

	ptList := TGeoDataObject.Create( ALayer, '', 0, Globe1.GlobalLayer.FindPresenter( 'GlobalPolylinePresenter', 0 ));
	ptList2 := TGeoDataObject.Create( ALayer, '', 0, ptList.Presenter );

	TPolylinePresenter( ptList.Presenter ).PolylinePen.PenColor := clRed;

	MyObject :=	TGeoDataObject.Create( ALayer, 'AirCraft', 0, ALayer.FindPresenter( 'GlobalPointPresenter', 0 ));

	Application.OnIdle := Idle;
end;

{------------------------------------------------------------------------------
	TForm1.Idle
------------------------------------------------------------------------------}
procedure TForm1.Idle(Sender: TObject; var Done: Boolean);
begin
	if giUpdateGlobe = 1 then
		with Globe2 do
		begin
			Invalidate;
			if Projection = gpSpherical then
			begin
				YRotation := Globe1.YRotation;
				XRotation := Globe1.XRotation;
			end;
		end;

	if giUpdateGlobe = 2 then
		with Globe1 do
			if Projection = gpSpherical then
			begin
				YRotation := Globe2.YRotation;
				XRotation := Globe2.XRotation;
			end;

	giUpdateGlobe := 0;
	Done := False;
end;

{------------------------------------------------------------------------------
	TForm1.Globe1MouseMove
------------------------------------------------------------------------------}
procedure TForm1.Globe1MouseMove( Sender : TObject; Shift : TShiftState; X,
	Y : Integer );
var
	obj : TGlobeObject;
	sTmp : string;
	pt : TPointLL;
begin
	Globe1.DeviceXYToLL( X, Y, pt );
	sTmp := Format( '%d,%d %d %d ', [ pt.iLongX, pt.iLatY,
		Globe1.GlobeCenterXY.X, Globe1.GlobeCenterXY.Y ] );

	obj := Globe1.ObjectAtXY( X, Y );
	if obj <> nil then
		sTmp := sTmp + obj.Title;
	Caption := 'TGlobe Version 3.00 Demo : ' + sTmp;
end;

{------------------------------------------------------------------------------
  TForm1.SpeedButton1Click
------------------------------------------------------------------------------}
procedure TForm1.SpeedButton1Click( Sender : TObject );
begin
	Globe1.Projection := TGlobeProjection( TSpeedButton( Sender ).Tag );
	Globe1.ScaleFactor := 0;

	with Globe2 do
	begin
		Projection := TGlobeProjection( TSpeedButton( Sender ).Tag );
		ScaleFactor := 0;
		if projection = gpSpherical then
			MouseRotateButton := mzbLeft
		else
			MouseRotateButton := mzbNone;
	end;
end;

{------------------------------------------------------------------------------
	TForm1.Globe1Render
------------------------------------------------------------------------------}
procedure TForm1.Globe1Render( Sender : TObject );
begin
	giUpdateGlobe := TGlobe3( Sender ).Tag;
end;

{------------------------------------------------------------------------------
	TForm1.Globe2Paint
------------------------------------------------------------------------------}
procedure TForm1.Globe2Paint( Sender : TObject );
var
	eW, eH : Extended;
	iX, iY : integer;
begin
	with Globe2 do
	begin
		eW := ( Globe1.Width / Globe1.ScaleFactor ) * ScaleFactor;
		eH := ( Globe1.Height / Globe1.ScaleFactor ) * ScaleFactor;
		iY := Round( Height / 2 - eH * ( Globe1.GlobeCenterXY.Y / Globe1.Height ) );
		iX := Round( Width / 2 - eW * ( Globe1.GlobeCenterXY.X / Globe1.Width ) );
		GlobeCanvas.Brush.Color := clRed;
		GlobeCanvas.FrameRect( Rect( iX, iY, iX + Round( eW ), iY + Round( eH )) );
	end;
end;

{------------------------------------------------------------------------------
  TForm1.Globe1Paint
------------------------------------------------------------------------------}
procedure TForm1.Globe1Paint( Sender : TObject );
begin
	if MyObject <> nil then
	begin
		Globe1.GlobeCanvas.Pen.color := clWhite;
		Globe1.RenderCircle( MyObject[0], GU_DEGREE * 10, 36 );

		ptList.Render;
		ptList2.Render;
	end;
end;

{------------------------------------------------------------------------------
	TForm1.DBGrid1CellClick
------------------------------------------------------------------------------}
procedure TForm1.DBGrid1CellClick( Column : TColumn );
var
	iIndex, idx, dPPI : integer;
	ptLL : TPointLL;
	dx, dy, dSF : Extended;
begin
	with Globe1 do
	begin
		iIndex := GlobeLayerDS1.FieldByName( 'Index' ).AsInteger;
		SelectedObject := GlobeLayerDS1.Layer.Objects[iIndex];

		GlobeCenterXY := Point( Width div 2, Height div 2 );
		DeviceXYtoLL( Width div 2, Height div 2, ptLL );

		ScaleFactor := 0;
		Application.ProcessMessages;

		with SelectedObject.Centroid do
		begin
			dx := ( iLongX - ptLL.iLongX ) / 10;
			dy := ( iLatY - ptLL.iLatY ) / 10;
		end;
//		with SelectedObject.BoundsRectLL do
//			dSF := ( MinExtended( Width / Right, Height / Bottom ) - ScaleFactor  ) / 5;

		for idx := 1 to 10 do
		begin
			LocatetoLL( ptLL.iLongX + Round( idx * dx ), ptLL.iLatY + Round( idx * dy ));
//			if idx > 5 then
//				ScaleFactor := ScaleFactor + dSF;
			Application.ProcessMessages;
		end;
	end;
end;

{------------------------------------------------------------------------------
	TForm1.DBGrid1TitleClick
------------------------------------------------------------------------------}
procedure TForm1.DBGrid1TitleClick( Column : TColumn );
begin
	if Column.FieldName = 'Index' then
		GlobeLayerDS1.SortColumn := lsIndex
	else
		GlobeLayerDS1.SortColumn := lsTitle;
end;

{------------------------------------------------------------------------------
	TForm1.btnZoomClick
------------------------------------------------------------------------------}
procedure TForm1.btnZoomClick( Sender : TObject );
begin
	with TSpeedButton( Sender ) do
		Globe1.Altitude := Round( Globe1.Altitude * ( Tag / 100 ));
end;

{------------------------------------------------------------------------------
  TForm1.btnViewTypeClick
------------------------------------------------------------------------------}
procedure TForm1.btnViewTypeClick( Sender : TObject );
begin
	with Globe1 do
		case TSpeedButton( Sender ).Tag of
			0 : { Normal }
				begin
					TextureBackground := False;
					TransparentGlobe := False;
					if LayerCount > 0 then
						Globe1[0].BrushOverride := nil;
				end;
			1 : { Transparent }
				begin
					TextureBackground := False;
					TransparentGlobe := True;
					if LayerCount > 0 then
						Globe1[0].BrushOverride := nil;
				end;
			2 : { Texturemapped }
				begin
					TransparentGlobe := False;
					if OpenDialog1.Execute then
						TextureFilename := OpenDialog1.FileName;
					TextureBackground := True;
					if TextureBackground and ( LayerCount > 0 ) then
						Globe1[0].BrushOverride := LayerBrush;
				end;
		end;
end;

{------------------------------------------------------------------------------
  TForm1.btnAboutClick
------------------------------------------------------------------------------}
procedure TForm1.btnAboutClick(Sender: TObject);
begin
	FormAbout.ShowModal;
end;

{------------------------------------------------------------------------------
	TForm1.Timer1Timer
------------------------------------------------------------------------------}
procedure TForm1.Timer1Timer(Sender: TObject);
const
	iStep : Extended = 0;
begin
	if MyObject <> nil then
	begin
		with MyObject[0] do
			MyObject[0] := PointLLH(
				( iLongX + GU_DEGREE ) mod GU_360_DEGREE,
				Round( 60 * GU_DEGREE * Cos( iStep )),
				5 * GU_DEGREE + Round( GU_DEGREE * 5 * Sin( iLongX * TORADIANS * 8 )));

		iStep := SphericalMod( iStep + LocalPi / 100 );

		ptList2.AddPointLL( PointLL( MyObject[0].iLongX, MyObject[0].iLatY ));
		ptList.AddPointLL( MyObject[0] );
		if ptList.CountLL > 100 then
		begin
			ptList.DeletePointLL( 0 );
			ptList2.DeletePointLL( 0 );
		end;
	end;
end;

end.

